#!/usr/bin/env node


// src/index.ts
import { Clerc, friendlyErrorPlugin, helpPlugin, notFoundPlugin, strictFlagsPlugin } from "clerc";

// package.json
var name = "create-vue-vine";
var version = "0.0.3";
var description = "Official CLI for creating Vue Vine project.";

// src/commands/create.ts
import { rm } from "node:fs/promises";
import { join as join2, relative } from "node:path";
import process3 from "node:process";

// src/create.ts
import { mkdir as mkdir2, writeFile } from "node:fs/promises";
import { join } from "node:path";

// src/utils/banner.ts
var gradientBanner = "\x1B[38;2;54;79;174mV\x1B[39m\x1B[38;2;54;81;172mu\x1B[39m\x1B[38;2;55;82;169me\x1B[39m \x1B[38;2;55;84;167mV\x1B[39m\x1B[38;2;55;86;164mi\x1B[39m\x1B[38;2;56;88;162mn\x1B[39m\x1B[38;2;56;89;160me\x1B[39m \x1B[38;2;56;91;157m-\x1B[39m \x1B[38;2;57;93;155mA\x1B[39m\x1B[38;2;57;94;152mn\x1B[39m\x1B[38;2;57;96;150mo\x1B[39m\x1B[38;2;57;98;147mt\x1B[39m\x1B[38;2;58;99;145mh\x1B[39m\x1B[38;2;58;101;143me\x1B[39m\x1B[38;2;58;103;140mr\x1B[39m \x1B[38;2;59;105;138ms\x1B[39m\x1B[38;2;59;106;135mt\x1B[39m\x1B[38;2;59;108;133my\x1B[39m\x1B[38;2;60;110;131ml\x1B[39m\x1B[38;2;60;111;128me\x1B[39m \x1B[38;2;60;113;126mo\x1B[39m\x1B[38;2;61;115;123mf\x1B[39m \x1B[38;2;61;117;121mw\x1B[39m\x1B[38;2;61;118;118mr\x1B[39m\x1B[38;2;62;120;116mi\x1B[39m\x1B[38;2;62;122;114mt\x1B[39m\x1B[38;2;62;123;111mi\x1B[39m\x1B[38;2;63;125;109mn\x1B[39m\x1B[38;2;63;127;106mg\x1B[39m \x1B[38;2;63;129;104mV\x1B[39m\x1B[38;2;64;130;102mu\x1B[39m\x1B[38;2;64;132;99me\x1B[39m \x1B[38;2;64;134;97mc\x1B[39m\x1B[38;2;64;135;94mo\x1B[39m\x1B[38;2;65;137;92mm\x1B[39m\x1B[38;2;65;139;89mp\x1B[39m\x1B[38;2;65;140;87mo\x1B[39m\x1B[38;2;66;142;85mn\x1B[39m\x1B[38;2;66;144;82me\x1B[39m\x1B[38;2;66;146;80mn\x1B[39m\x1B[38;2;67;147;77mt\x1B[39m\x1B[38;2;67;149;75ms\x1B[39m";

// src/utils/clack.ts
import process from "node:process";
import { cancel as _cancel, confirm as _confirm, text as _text, isCancel } from "@clack/prompts";
function cancel(...args) {
  _cancel(...args);
  process.exit(0);
}
function wrapClack(fn) {
  return async (...args) => {
    const result = await fn(...args);
    if (isCancel(result)) {
      cancel("Operation cancelled. Goodbye!");
      process.exit(0);
    }
    return result;
  };
}
var text = wrapClack(_text);
var confirm = wrapClack(_confirm);

// src/utils/flags.ts
function defineFlagMeta(flag) {
  return flag;
}

// src/utils/fs.ts
import { access } from "node:fs/promises";
function exists(path2) {
  return access(path2).then(() => true).catch(() => false);
}

// src/utils/pm.ts
import process2 from "node:process";
import { execa, execaSync } from "execa";
var userAgentEnv = process2.env.npm_config_user_agent ?? "";
function detectPackageManager() {
  const fromUserAgent = /pnpm/.test(userAgentEnv) ? "pnpm" : /yarn/.test(userAgentEnv) ? /bun/.test(userAgentEnv) ? "bun" : "yarn" : void 0;
  if (fromUserAgent)
    return fromUserAgent;
  let pnpmVersionExitCode, yarnVersionExitCode;
  try {
    pnpmVersionExitCode = execaSync("pnpm", ["--version"], { stdio: "ignore" }).exitCode;
  } catch {
    pnpmVersionExitCode = -1;
  }
  try {
    yarnVersionExitCode = execaSync("yarn", ["--version"], { stdio: "ignore" }).exitCode;
  } catch {
    yarnVersionExitCode = -1;
  }
  if (pnpmVersionExitCode === 0) {
    return "pnpm";
  } else if (yarnVersionExitCode === 0) {
    return "yarn";
  }
  return "npm";
}
var packageManager = detectPackageManager();
function getPmCommand(command) {
  let commands;
  if (packageManager === "pnpm") {
    commands = ["pnpm", [command]];
  } else {
    commands = [packageManager, [command === "install" ? "" : "run", command].filter(Boolean)];
  }
  return commands;
}
function formatPmCommand(commands) {
  return commands.flat().join(" ");
}
function runPmCommand(command, cwd) {
  return execa(...getPmCommand(command), { stdio: "inherit", cwd });
}

// src/utils/project.ts
import { basename, dirname, resolve } from "node:path";
import { fileURLToPath } from "node:url";
function validateProjectName(path2) {
  return basename(path2) === path2;
}
var __filename = fileURLToPath(import.meta.url);
var __dirname = dirname(__filename);
async function getTemplateDirectory() {
  const templateRoot = resolve(__dirname, "../template");
  return await exists(templateRoot) ? templateRoot : void 0;
}

// src/utils/renderTemplate.ts
import fs from "node:fs";
import { copyFile, mkdir, readdir, stat } from "node:fs/promises";
import path from "node:path";

// src/utils/deepMerge.ts
var isObject = (val) => val && typeof val === "object";
var mergeArrayWithDedupe = (a, b) => Array.from(/* @__PURE__ */ new Set([...a, ...b]));
function deepMerge(target, obj) {
  for (const key of Object.keys(obj)) {
    const oldVal = target[key];
    const newVal = obj[key];
    if (Array.isArray(oldVal) && Array.isArray(newVal)) {
      target[key] = mergeArrayWithDedupe(oldVal, newVal);
    } else if (isObject(oldVal) && isObject(newVal)) {
      target[key] = deepMerge(oldVal, newVal);
    } else {
      target[key] = newVal;
    }
  }
  return target;
}
var deepMerge_default = deepMerge;

// src/utils/sortDependencies.ts
function sortDependencies(packageJson) {
  const sorted = {};
  const depTypes = ["dependencies", "devDependencies", "peerDependencies", "optionalDependencies"];
  for (const depType of depTypes) {
    if (packageJson[depType]) {
      sorted[depType] = {};
      Object.keys(packageJson[depType]).sort().forEach((name2) => {
        sorted[depType][name2] = packageJson[depType][name2];
      });
    }
  }
  return {
    ...packageJson,
    ...sorted
  };
}

// src/utils/renderTemplate.ts
async function renderTemplate(src, dest) {
  const stats = await stat(src);
  if (stats.isDirectory()) {
    await mkdir(dest, { recursive: true });
    for (const file of await readdir(src)) {
      renderTemplate(path.resolve(src, file), path.resolve(dest, file));
    }
    return;
  }
  const filename = path.basename(src);
  if (filename.startsWith("_")) {
    dest = path.resolve(path.dirname(dest), `.${filename.slice(1)}`);
  }
  if (filename === "package.json" && fs.existsSync(dest)) {
    const existing = JSON.parse(fs.readFileSync(dest, "utf8"));
    const newPackage = JSON.parse(fs.readFileSync(src, "utf8"));
    const pkg = sortDependencies(deepMerge_default(existing, newPackage));
    fs.writeFileSync(dest, `${JSON.stringify(pkg, null, 2)}
`);
    return;
  }
  await copyFile(src, dest);
}

// src/create.ts
var builtInTemplates = [
  "common",
  "code/base",
  "config/ts",
  "config/eslint"
];
function createProjectOptions(params) {
  return {
    ...params,
    templates: []
  };
}
async function createProject(options) {
  const templateDirectory = await getTemplateDirectory();
  const withBase = (path2) => join(templateDirectory, path2);
  await mkdir2(options.path);
  await writeFile(join(options.path, "package.json"), JSON.stringify({
    name: options.name
  }, null, 2));
  for (const template of [
    ...builtInTemplates,
    ...options.templates
  ]) {
    await renderTemplate(withBase(template), options.path);
  }
}

// src/flags/index.ts
var metas = {
  router: defineFlagMeta({
    name: "router",
    message: "Use Vue Router?",
    flag: {
      type: Boolean,
      description: "Add Vue Router",
      default: false
    },
    initialValue: false
  })
};
var flags = Object.entries(metas).reduce((acc, [key, value]) => {
  Reflect.set(acc, key, value.flag);
  return acc;
}, {});
function useFlags() {
  return {
    flags,
    executeFlags: async (flags3, options) => {
      const context = {
        template: (...path2) => {
          options.templates.push(...path2);
        }
      };
      for (const item of [metas.router.name]) {
        if (!flags3[item]) {
          const { initialValue, message } = metas[item];
          flags3[item] = await confirm({
            message,
            initialValue
          });
        }
      }
      if (flags3.router) {
        context.template("code/router", "config/router");
      }
    }
  };
}

// src/commands/create.ts
import { intro, log, outro, spinner } from "@clack/prompts";
import { defineCommand, Root } from "clerc";
import { bold, green } from "yoctocolors";
var defaultProjectName = "vue-vine-project";
var { flags: flags2, executeFlags } = useFlags();
var createCommand = defineCommand({
  name: Root,
  description: "Create a Vue Vine project",
  parameters: [
    "[projectName]"
  ],
  flags: {
    force: {
      type: Boolean,
      description: "Delete existing folder",
      alias: "f",
      default: false
    },
    install: {
      type: Boolean,
      description: "Install dependencies",
      alias: "i"
    },
    ...flags2
  },
  alias: "create"
}, async (ctx) => {
  intro(gradientBanner);
  const cwd = process3.cwd();
  if (!ctx.parameters.projectName) {
    ctx.parameters.projectName = await text({
      message: "Project name:",
      placeholder: defaultProjectName,
      defaultValue: defaultProjectName,
      validate: (value) => {
        if (!validateProjectName(value)) {
          return "Invalid project name";
        }
      }
    });
  }
  const projectPath = join2(cwd, ctx.parameters.projectName);
  if (await exists(projectPath)) {
    if (!ctx.flags.force) {
      ctx.flags.force = await confirm({
        message: `Folder ${ctx.parameters.projectName} already exists. Delete?`,
        initialValue: false
      });
    }
    if (!ctx.flags.force) {
      cancel(`Folder ${ctx.parameters.projectName} already exists. Goodbye!`);
    } else {
      log.info(`Folder ${ctx.parameters.projectName} will be deleted.`);
      await rm(projectPath, { recursive: true });
    }
  }
  const templateDir = await getTemplateDirectory();
  if (!templateDir) {
    cancel("Unable to find template directory");
  }
  const projectOptions = createProjectOptions({
    path: projectPath,
    name: ctx.parameters.projectName,
    templateDir
  });
  await executeFlags(ctx.flags, projectOptions);
  const s = spinner();
  s.start(`Creating project ${ctx.parameters.projectName}`);
  await createProject(projectOptions);
  s.stop(`Project created at: ${projectPath}`);
  if (ctx.flags.install === void 0) {
    ctx.flags.install = await confirm({
      message: "Install dependencies?",
      initialValue: true
    });
  }
  if (ctx.flags.install) {
    s.start("Installing dependencies");
    await runPmCommand("install", projectPath);
    s.stop("Dependencies installed!");
  }
  const cdProjectPath = relative(cwd, projectPath);
  const helpText = [
    "You're all set! Now run:",
    "",
    `  cd ${bold(green(cdProjectPath.includes(" ") ? `"${cdProjectPath}"` : cdProjectPath))}`,
    ctx.flags.install ? void 0 : `  ${bold(green(formatPmCommand(getPmCommand("install"))))}`,
    `  ${bold(green(formatPmCommand(getPmCommand("dev"))))}`,
    "",
    "  Happy hacking!"
  ].filter((s2) => s2 !== void 0).join("\n");
  outro(helpText);
  process3.exit();
});

// src/index.ts
Clerc.create(name, version, description).use(helpPlugin()).use(notFoundPlugin()).use(strictFlagsPlugin()).use(notFoundPlugin()).use(friendlyErrorPlugin()).command(createCommand).parse();
